//
// kernel.cpp
//
#include "kernel.h"

static const char FromKernel[] = "kernel";

CKernel::CKernel (void)
:	m_Screen (m_Options.GetWidth (), m_Options.GetHeight ()),
	m_Timer (&m_Interrupt),
	m_Logger (m_Options.GetLogLevel (), &m_Timer),
	m_OneWire (24),			// on GPIO pin 24 (a 4.7K resistor is necessary)
	m_DS18x20 (&m_OneWire)		// with parasite power
{
	m_ActLED.Blink (5);	// show we are alive
}

CKernel::~CKernel (void)
{
}

boolean CKernel::Initialize (void)
{
	boolean bOK = TRUE;

	if (bOK)
	{
		bOK = m_Screen.Initialize ();
	}

	if (bOK)
	{
		bOK = m_Serial.Initialize (115200);
	}

	if (bOK)
	{
		CDevice *pTarget = m_DeviceNameService.GetDevice (m_Options.GetLogDevice (), FALSE);
		if (pTarget == 0)
		{
			pTarget = &m_Screen;
		}

		bOK = m_Logger.Initialize (pTarget);
	}

	if (bOK)
	{
		bOK = m_Interrupt.Initialize ();
	}

	if (bOK)
	{
		bOK = m_Timer.Initialize ();
	}

	if (bOK)
	{
		bOK = m_DS18x20.Initialize ();
	}

	return bOK;
}

TShutdownMode CKernel::Run (void)
{
	m_Logger.Write (FromKernel, LogNotice, "Compile time: " __DATE__ " " __TIME__);

	while (1)
	{
		if (m_DS18x20.StartConversion ())
		{
			m_Logger.Write (FromKernel, LogNotice, "Temperature is %d.%dC / %d.%dF",
					m_DS18x20.GetCelsius (),    m_DS18x20.GetCelsiusFract (), 
					m_DS18x20.GetFahrenheit (), m_DS18x20.GetFahrenheitFract ());
		}
		else
		{
			m_Logger.Write (FromKernel, LogWarning, "Conversion failed");
		}
	}

	return ShutdownHalt;
}
